/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined(BRICONWIDGET_H)
#define BRICONWIDGET_H

#if defined(WIN32)
#pragma warning (disable:4786)
#endif

#include <qwidget.h>
#include <list>

#include "bricon.h"
#include "overlaylist.h"

class QLineEdit;
class QToolButton;
class QSlider;
class BriConWidget;
class BriConWheel;

class BriConWidgetObserver
{
public:

  virtual ~BriConWidgetObserver() {}
  virtual void update(const BriConWidget* w) = 0;

  BriConWidgetObserver() {}
};

class BriConWidget : public QWidget, OverlayListObserver
{
  Q_OBJECT
public:
  BriConWidget(QWidget *parent, OverlayList::Handle list);
  virtual ~BriConWidget();

  virtual void update(const OverlayList* list, OverlayListMsg message);

  void attach(BriConWidgetObserver* o);
  void detach(BriConWidgetObserver* o);
  void notify();

private:
  
  BriCon::Handle m_bricon;
  OverlayList::Handle   m_list;
  bool         m_blockEvents;
  QLineEdit*   m_minBox;
  QLineEdit*   m_maxBox;
  BriConWheel* m_briSlider;
  BriConWheel* m_conSlider;
  float        m_originalMin;
  float        m_originalMax;
  void         updateMinMaxBoxes();

  std::list<BriConWidgetObserver*> m_observers;
  
private slots:

  void reset();
  void minChanged(const QString& valueText);
  void maxChanged(const QString& valueText);
  void briSliderChanged(double value);
  void conSliderChanged(double value);
  void sliderReleased();
};

#endif
