/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined(IMAGEWIDGET_H)
#define IMAGEWIDGET_H

#include <qtimer.h>
#include "drawwidget.h"
#include "viewwidget.h"
#include "overlaylist.h"
#include "shape.h"
#include "cursor.h"

class QToolBar;
class QToolButton;
class QSpinBox;
class QCheckBox;
class OverlayWidget;
class BriConWidget;
class CursorWidget;
class DrawWidget;
class ImageGroup;
class OverlayDialog;
class QTimer;



class ImageWidget : public ViewWidget, public DrawWidgetObserver,
                    public OverlayListObserver
{
  Q_OBJECT

public:
  ImageWidget(QWidget* parent,ImageGroup::Handle i,
              OverlayList::Handle,Cursor::Handle c);
  ~ImageWidget();  
  OverlayList::Handle getOverlayList(); 
  void update(const DrawWidget* w);
  void update(const OverlayList* i, OverlayListMsg msg);
  virtual void  update(const Cursor::Handle& c);

  inline int inqX();
  inline int inqY();
  inline int inqZ();

public slots:
  void setPanMode();
  void setZoomMode(); 
  void setCursorMode();
  void setMaskMode();
  void setCrossHairMode();
  void setZoomCombo(int);  
  void setVolSpinBox(int);
  void openOverlayDialog();
  void volChanged(int n);

private slots: 
  void maskButtonToggled(bool);
  void toggleMovie();  
  void nextFrame();

signals:
  virtual void zoomChanged(int n);
  virtual void resetZoom();
  virtual void crossHairModeChanged(bool);

private:
  void connectControls();
  void constructToolBar(); 
  void loadOverlaysList();
  void constructor();
  void undoGraphics();
  void clearUndoList();
  void dtiDisplayMode(int);

  QSpinBox            *m_zoomSpinBox;
  QSpinBox            *m_volSpinBox;  
  QSpinBox            *m_speedSpinBox;
  QTimer              *m_movieTimer;
  QToolButton         *m_movieButton;

  ImageGroup::Handle   m_imageGroup; 

  int m_xDim;
  int m_yDim; 
  int m_zDim;

protected:
  QToolButton         *m_noneModeButton;
  Cursor::Handle       m_cursor;
  QToolButton         *m_zoomModeButton;
  QToolButton         *m_cursorModeButton;
  QToolButton         *m_panModeButton;
  QToolButton         *m_maskModeButton;
  QToolButton         *m_resetButton;
  QCheckBox           *m_crossHairCheckBox;
  OverlayList::Handle  m_overlayList;  
  OverlayWidget       *m_overlayWidget;
  BriConWidget        *m_briconWidget;
  CursorWidget        *m_cursorWidget;
  DrawWidget          *m_drawWidget;
  OverlayDialog       *m_overlayDialog;
  QToolBar            *m_toolbar; 
  std::list<Shape::Handle>        m_undoList;
};

int ImageWidget::inqX(){return m_xDim;}
int ImageWidget::inqY(){return m_yDim;}
int ImageWidget::inqZ(){return m_zDim;}
#endif
