/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined(MODELFIT_H)
#define MODELFIT_H
#include "newmatap.h"
#include <boost/shared_ptr.hpp>
#include "miscmaths/miscmaths.h"
#include "storage/image.h"
#include <qstring.h>
#include "tracker.h"
#include <vector.h>
#include "stdio.h"


using namespace NEWMAT;
using namespace MISCMATHS;

class ModelFit
{
public:
  typedef boost::shared_ptr< ModelFit > Handle;
  static Handle create(const QString & featDir);
  virtual ~ModelFit(); 

  TimeSeries::Handle     getModelTimeSeries(short x, short y, short z,
                                            float offset);
  TimeSeries::Handle     getDataTimeSeries(short x, short y, short z);

private:  
  TimeSeries::Handle     preWhitenTimeseries(const ColumnVector& ac, 
                                             TimeSeries::Handle& ts);
  QString  findKey(FILE *fd, const QString & keyName);
  void     readModel(const QString & filePath);
  void     loadThresAc(const QString & filePath);
  void     loadFiltFuncData(const QString & filePath);
  void     loadPeData(const QString & filePath,int npts,int nevs);
  ColumnVector getPeColumnVector(short x, short y, short z);
  ColumnVector getThrColumnVector(short x, short y, short z);
  ModelFit(const QString & featDir); 
  Matrix m_designMatrix;
  std::vector<Volume::Handle> m_peData;
  Image::Handle m_threshAc;
  Image::Handle m_filtFuncData;

  TimeSeries::Handle m_timeSeriesModel;
  QString m_featDir;
};

#endif
