/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if !defined (OVERLAYWIDGET_H)
#define OVERLAYWIDGET_H

#include <algorithm>
#include <qwidget.h>
#include <qlistview.h>
#include "overlaylist.h"
#include "metaimage.h"


class QCheckBox;
class QSlider;
class QPushButton;
class LayerListItem;
class ImageWidget;


class OverlayWidget : public QWidget, OverlayListObserver
{

 Q_OBJECT

 public:
  OverlayWidget(QWidget* w,OverlayList::Handle l);
  virtual ~OverlayWidget();
  void update(const OverlayList* l, OverlayListMsg msg);
 private:  
  void updateListView();
  void updateListItem();
  void setAllInActive(); 
  void updateUpDownButtons();
  void setVisibility(bool state);
  void updateControls();
  QListViewItem* getLayerItem(const MetaImage::Handle mi)const;
  

  OverlayList::Handle   m_overlayList; 
  QCheckBox*     m_visibleButton;
  QCheckBox*     m_unlockButton;
  QListView*     m_layersListView;
  QSlider*       m_transSlider;
  QSlider*       m_modTransSlider;
  QPushButton*   m_upButton;
  QPushButton*   m_downButton;

  bool           m_blockEvents;
  bool           m_blockSliderUpdate;

 signals:
   virtual void infoButtonAction();

 private slots:

     void listSelectChanged();
     void visibleButtonChanged(bool state);
     void unlockButtonChanged(bool state);
 
     void listDoubleClicked(QListViewItem* item);

     void transSliderChanged(int value);     
     void modTransSliderChanged(int value);
     void upButtonPressed();
     void downButtonPressed();
     void detailsButtonPressed();
};


#endif
