#

# clusterrep 

# Copyright (C) 1999 John O'Doherty
# Dept. of Exp. Psy., Oxford. E-mail:john.odoherty@psy.ox.ac.uk

#   Part of FSL - FMRIB's Software Library
#   http://www.fmrib.ox.ac.uk/fsl
#   fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   
#   LICENCE
#   
#   FMRIB Software Library, Release 3.2beta (c) 2004, The University of
#   Oxford (the "Software")
#   
#   The Software remains the property of the University of Oxford ("the
#   University").
#   
#   The Software is distributed "AS IS" under this Licence solely for
#   non-commercial use in the hope that it will be useful, but in order
#   that the University as a charitable foundation protects its assets for
#   the benefit of its educational and research purposes, the University
#   makes clear that no condition is made or to be implied, nor is any
#   warranty given or to be implied, as to the accuracy of the Software,
#   or that it will be suitable for any particular purpose or for use
#   under any specific conditions. Furthermore, the University disclaims
#   all responsibility for the use which is made of the Software. It
#   further disclaims any liability for the outcomes arising from using
#   the Software.
#   
#   The Licensee agrees to indemnify the University and hold the
#   University harmless from and against any and all claims, damages and
#   liabilities asserted by third parties (including claims for
#   negligence) which arise directly or indirectly from the use of the
#   Software or the sale of any products based on the Software.
#   
#   No part of the Software may be reproduced, modified, transmitted or
#   transferred in any form or by any means, electronic or mechanical,
#   without the express permission of the University. The permission of
#   the University is not required if the said reproduction, modification,
#   transmission or transference is done without financial return, the
#   conditions of this Licence are imposed upon the receiver of the
#   product, and all original and amended source code is included in any
#   transmitted product. You may be held legally responsible for any
#   copyright infringement that is caused or encouraged by your failure to
#   abide by these terms and conditions.
#   
#   You are not permitted under this Licence to use this Software
#   commercially. Use for which any financial return is received shall be
#   defined as commercial use, and includes (1) integration of all or part
#   of the source code or the Software into a product for sale or license
#   by or on behalf of Licensee to third parties or (2) use of the
#   Software or any derivative of it for research with the final aim of
#   developing software products for sale or license to a third party or
#   (3) use of the Software or any derivative of it for research with the
#   final aim of developing non-software products for sale or license to a
#   third party, or (4) use of the Software to provide any service to an
#   external organisation for which payment is received. If you are
#   interested in using the Software commercially, please contact Isis
#   Innovation Limited ("Isis"), the technology transfer company of the
#   University, to negotiate a licence. Contact details are:
#   innovation@isis.ox.ac.uk quoting reference DE/1112.

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

set outname "/tmp/${USER}cluster.txt"

MxSetTextDefaults {{OutlineColor none} {FillColor none} {TextColor green} {HorzJustify Center} {VertJustify Bottom} {NameFont -adobe-courier-bold-r-normal--8-80-75-75-m-50-iso8859-1}}

#Ask user to input 3 page names: (1) thresholded z-score (2) % change map name
#(3) fused reference map (4) cluster output name

InputFromWidget "Cluster output" 5 "Thresholded z-score image" zimage 20 "%change activation image" actimg 20 "Fused reference image" refimg 20 "Image Mask" ImageMask  20 "Cluster output file" outname 20

set annotate [YesNoWidget "Annotate Fused Image with Cluster numbers?" Yes No]


MxGetCurrentFolder Folder
MxGetPageByName $Folder $ImageMask Mask

#First get z-score image and threshold it
MxGetPageByName $Folder $zimage zpage

# Make sure that no blobs touch the edges - MEDx cluster detection
# doen't work if they do. Then make real
MxGetImageProperties $zpage pl
set bpp [keylget pl Bits]
set ww [keylget pl Width]
set hh [keylget pl Height]
set ss [keylget pl Slices]
set wwh [expr ( $ww - 1 )/2]
set hhh [expr ( $hh - 1 )/2]
set ssh [expr ( $ss - 1 )/2]
incr ww 10
incr hh 10
incr ss 4
keylset newlist Cx $wwh Cy $hhh Cz $ssh Rx 0 Ry 0 Rz 0 Sx 1 Sy 1 Sz 1 Tx 0 Ty 0 Tz 0 Dx $ww Dy $hh Dz $ss RotMethod M
MxInterpolationMode NN
MxReslice $zpage newlist zpagedup
MxGetPageByName $Folder $actimg actpag
MxReslice $actpag newlist actpagedup
MxGetPageByName $Folder $refimg refpage
MxSaveLUT $refpage "/tmp/${USER}templut"
MxReslice $refpage newlist refpagedup
MxReslice $Mask newlist maskdup

MxThreshold $zpagedup 1 B1

#set to 2DALL because of bug in Digital v. of MEDX
MxSetImageOpMode $actpagedup 2DAll

MxExpr "{padding ZP} {InOut In} {Bounds C} {gain 1} {KSize 3} {low 0} {high 1} {AllSlices true}" $actpagedup = "($actpagedup * $zpagedup)"
MxSetImageOpMode $actpagedup 3D
MxDeletePage $zpagedup

# Steve Smiths's trick from cluster_count_procs.tcl!
# Fool medx into thinking that this is a z score image so it doesn't 
# prompt with the "did you know this isn't a z score?" question
#MxDupPage $actpagedup actpagedup2
#MxGroupImages { $actpagedup $actpagedup2 } I "New Group1" NG1
#MxBetweenGrpStatistics $NG1 $NG1 U I0 I2 I3 I4 I5
#MxExpr $I4 = $actpagedup

#then run clusteranalysis
MxClusterDetection $actpagedup $maskdup 0.01 1 $outname cluster probability resultgroup

MxDeletePage {$maskdup $actpagedup} 
MxSetCurrentPage $refpagedup
MxLoadLUT $refpagedup "/tmp/${USER}templut"

if {$annotate==1} {

    set channel [open $outname "r"]
    set found 0
    set noexit 1 
    MxGetCurrentImage image

    while {$noexit} {

	gets $channel line
	set element [lindex $line 0]

	if  {$element=="Cluster#"} {
	    #found start of the business
	    while {$line!=""} {
		gets $channel line
		set clustno [lindex $line 0]
		set size    [lindex $line 1]
		set x	    [lindex $line 3]
		set y 	    [lindex $line 4]
		set z	    [lindex $line 5]
		if {$size>2} {
		    if {$clustno<10} {
		    MxDrawTextGraphic $image $clustno $x $y $z 12 8 text
		    } else {
		    MxDrawTextGraphic $image $clustno $x $y $z 16 8 text
		    }
		} 
	    }
	    set noexit 0
	}
	
    }
 }
#end annotate if


#OK then place 3D Marker 

while (1) {

MxSetGraphicToolMode 3DMarker
set reply [YesNoWidget "Move to cluster location click Report" Report Exit]
if {$reply == 0} {
 break }

MxGetCurrentGraphic marker
MxGet3DMarkerProperties $marker Properties
set mx [keylget Properties X]
set my [keylget Properties Y]
set mz [keylget Properties Z]

MxGetVoxel $cluster $mx $my $mz value
ScriptUpdate "The cluster number is : $value"
MxDeleteGraphic $marker

}

