#

#   concat_transforms - concatenate two registration transofmrations
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   http://www.fmrib.ox.ac.uk/fsl
#   fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   
#   LICENCE
#   
#   FMRIB Software Library, Release 3.2beta (c) 2004, The University of
#   Oxford (the "Software")
#   
#   The Software remains the property of the University of Oxford ("the
#   University").
#   
#   The Software is distributed "AS IS" under this Licence solely for
#   non-commercial use in the hope that it will be useful, but in order
#   that the University as a charitable foundation protects its assets for
#   the benefit of its educational and research purposes, the University
#   makes clear that no condition is made or to be implied, nor is any
#   warranty given or to be implied, as to the accuracy of the Software,
#   or that it will be suitable for any particular purpose or for use
#   under any specific conditions. Furthermore, the University disclaims
#   all responsibility for the use which is made of the Software. It
#   further disclaims any liability for the outcomes arising from using
#   the Software.
#   
#   The Licensee agrees to indemnify the University and hold the
#   University harmless from and against any and all claims, damages and
#   liabilities asserted by third parties (including claims for
#   negligence) which arise directly or indirectly from the use of the
#   Software or the sale of any products based on the Software.
#   
#   No part of the Software may be reproduced, modified, transmitted or
#   transferred in any form or by any means, electronic or mechanical,
#   without the express permission of the University. The permission of
#   the University is not required if the said reproduction, modification,
#   transmission or transference is done without financial return, the
#   conditions of this Licence are imposed upon the receiver of the
#   product, and all original and amended source code is included in any
#   transmitted product. You may be held legally responsible for any
#   copyright infringement that is caused or encouraged by your failure to
#   abide by these terms and conditions.
#   
#   You are not permitted under this Licence to use this Software
#   commercially. Use for which any financial return is received shall be
#   defined as commercial use, and includes (1) integration of all or part
#   of the source code or the Software into a product for sale or license
#   by or on behalf of Licensee to third parties or (2) use of the
#   Software or any derivative of it for research with the final aim of
#   developing software products for sale or license to a third party or
#   (3) use of the Software or any derivative of it for research with the
#   final aim of developing non-software products for sale or license to a
#   third party, or (4) use of the Software to provide any service to an
#   external organisation for which payment is received. If you are
#   interested in using the Software commercially, please contact Isis
#   Innovation Limited ("Isis"), the technology transfer company of the
#   University, to negotiate a licence. Contact details are:
#   innovation@isis.ox.ac.uk quoting reference DE/1112.

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

# Summary:      Creates a file select dialog box.
proc ConcTrans { w } {
    global ConcTrans
 
    if [winfo exists $w] {
        wm deiconify $w
        raise $w
        return
    }
 
    toplevel $w
    wm title $w "Concat two transforms"
    wm iconname $w "Concat two transforms"
 
    frame $w.f
 
    tixFileEntry $w.filename  -label "First Transform:"\
        -variable ConcTrans($w,filename) \
        -options {
            entry.width 40
        }
    $w.filename filedialog subwidget fsbox config -directory ~
    pack $w.filename -in $w.f -side top -expand yes -fill both \
        -padx 3 -pady 10

    tixOptionMenu $w.format -label "First Transform Type:" \
        -variable ConcTrans($w,format) \
        -options {
            label.anchor w
            menubutton.width 20
    }
    OpMenu $w.format [list Talairach AIR Reslice Resample User]
    set ConcTrans($w,format) AIR
    $w.format configure -value $ConcTrans($w,format)
    pack $w.format -in $w.f -side top -padx 3 -pady 3

    tixFileEntry $w.filename2  -label "Second Transform:"\
        -variable ConcTrans($w,filename2) \
        -options {
            entry.width 40
        }
    $w.filename2 filedialog subwidget fsbox config -directory ~
    pack $w.filename2 -in $w.f -side top -expand yes -fill both \
        -padx 3 -pady 10

    tixOptionMenu $w.format2 -label "Second Transform Type:" \
        -variable ConcTrans($w,format2) \
        -options {
            label.anchor w
            menubutton.width 20
    }
    OpMenu $w.format2 [list Talairach AIR Reslice Resample User]
    set ConcTrans($w,format2) Talairach
    $w.format2 configure -value $ConcTrans($w,format2)
    pack $w.format2 -in $w.f -side top -padx 3 -pady 3

    tixFileEntry $w.filename3  -label "Output Transform:"\
        -variable ConcTrans($w,filename3) \
        -options {
            entry.width 40
        }
    $w.filename3 filedialog subwidget fsbox config -directory ~
    pack $w.filename3 -in $w.f -side top -expand yes -fill both \
        -padx 3 -pady 10

    # ---- Status Bar ----
 
    label $w.status -height 1 -relief sunken -bd 2
 
    #------------- Button Frame --------------

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
 
    button $w.ok \
        -text "OK" -width 5 \
        -command "ConcTrans:apply $w destroy"
    bind $w.ok <Return> {
        [winfo toplevel %W].ok invoke}
 
    button $w.apply     -command "ConcTrans:apply $w keep" \
        -text "Apply" -width 5
    bind $w.apply <Return> {
        [winfo toplevel %W].apply invoke}
 
    button $w.cancel    -command "ConcTrans:destroy $w" \
        -text "Cancel" -width 5
    bind $w.cancel <Return> {
        [winfo toplevel %W].cancel invoke}
 
    pack $w.btns.b -side bottom -fill x -padx 3 -pady 5
    pack $w.ok $w.apply $w.cancel -in $w.btns.b \
        -side left -expand yes -padx 3 -pady 10 -fill y
 
    pack $w.f $w.status $w.btns -expand yes -fill both
}
 
 
#
# Procedure:    ConcTrans:apply
#
# Usage:        ConcTrans:apply widget destroy/keep
#
# Summary:      Creates a file select dialog box.
#
proc ConcTrans:apply { w dialog } {
    global ConcTrans

    $w.filename update
    $w.filename2 update
    $w.filename3 update

    set result [MxConcatTransforms $ConcTrans($w,filename) $ConcTrans($w,format) \
                    $ConcTrans($w,filename2) $ConcTrans($w,format2) $ConcTrans($w,filename3)]

    if {$result} {
      MxPause "Transform types are incorrect for the chosen transforms"
    } else {
      # Let the user know were done
      $w.status configure -text "Done"
      update idletasks

      # Get rid of the dialog box if OK button used
      if {$dialog == "destroy"} {
          ConcTrans:destroy $w
      }
    }
}

# Summary:      Destroys ConcTrans dialog box
proc ConcTrans:destroy { w } {
 
        destroy $w
}

ConcTrans .rename
tkwait window .rename

